@@ -35,7 +35,7 @@
 #endif
 
 #if !defined(MG_ARCH)
-#include <mongoose_custom.h>
+#include "mongoose_custom.h"  // keep this include
 #endif
 
 #if !defined(MG_ARCH)
