@@ -2490,9 +2490,36 @@ static void test_json(void) {
     ASSERT((str = mg_json_get_hex(json, "$[1]", &len)) != NULL);
     ASSERT(len == 5 && memcmp(str, "\x01\x00\x02hi", (size_t) len) == 0);
     free(str);
+
+    json = mg_str("{\"a\":[1,2,3], \"ab\": 2}");
+    ASSERT(mg_json_get_long(json, "$.a[0]", -42) == 1);
+    ASSERT(mg_json_get_long(json, "$.ab", -42) == 2);
+    ASSERT(mg_json_get_long(json, "$.ac", -42) == -42);
+
+    json = mg_str("{\"a\":[],\"b\":[1,2]}");
+    ASSERT(mg_json_get_long(json, "$.a[0]", -42) == -42);
+    ASSERT(mg_json_get_long(json, "$.b[0]", -42) == 1);
+    ASSERT(mg_json_get_long(json, "$.b[1]", -42) == 2);
+    ASSERT(mg_json_get_long(json, "$.b[2]", -42) == -42);
+    json = mg_str("[{\"a\":1,\"b\":2},{\"a\":3, \"b\":4}]");
+    ASSERT(mg_json_get_long(json, "$[0].a", -42) == 1);
+    ASSERT(mg_json_get_long(json, "$[0].b", -42) == 2);
+    ASSERT(mg_json_get_long(json, "$[1].a", -42) == 3);
+    ASSERT(mg_json_get_long(json, "$[1].b", -42) == 4);
+    ASSERT(mg_json_get_long(json, "$[2].a", -42) == -42);
+
+    json = mg_str("{\"a\":[1],\"b\":[2,3]}");
+    ASSERT(mg_json_get_long(json, "$.a[0]", -42) == 1);
+    ASSERT(mg_json_get_long(json, "$.a[1]", -42) == -42);
+
   }
 }
 
+static void resp_rpc(struct mg_rpc_req *r) {
+  int len = 0, off = mg_json_get(r->frame, "$.result", &len);
+  mg_xprintf(r->pfn, r->pfn_data, "%.*s", len, &r->frame.ptr[off]);
+}
+
 static void test_rpc(void) {
   struct mg_rpc *head = NULL;
   struct mg_iobuf io = {0, 0, 0, 256};
@@ -2541,10 +2568,30 @@ static void test_rpc(void) {
     const char *resp = "{\"error\":{\"code\":-32700,\"message\":\"haha\"}}";
     req.frame = mg_str("haha");
     mg_rpc_process(&req);
+    // MG_INFO(("-> %s", io.buf));
     ASSERT(strcmp((char *) io.buf, resp) == 0);
     mg_iobuf_free(&io);
   }
 
+  {
+    const char *resp =
+        "{\"id\":1,\"error\":{\"code\":-32601,\"message\":\" not found\"}}";
+    req.frame = mg_str("{\"id\":1,\"result\":123}");
+    mg_rpc_process(&req);
+    // MG_INFO(("-> %s", io.buf));
+    ASSERT(strcmp((char *) io.buf, resp) == 0);
+    mg_iobuf_free(&io);
+  }
+
+  {
+    req.frame = mg_str("{\"id\":1,\"result\":123}");
+    mg_rpc_add(&head, mg_str(""), resp_rpc, NULL);
+    mg_rpc_process(&req);
+    MG_INFO(("-> %s", io.buf));
+    ASSERT(strcmp((char *) io.buf, "123") == 0);
+    mg_iobuf_free(&io);
+  }
+
   mg_rpc_del(&head, NULL);
   ASSERT(head == NULL);
 }
